@@ -14,12 +14,12 @@
 #include <limits.h>
 #include <math.h>
 #include <string.h>
+#include <stdint.h>
 
 #include "analyze-linkage.h"
 #include "corpus/corpus.h"
 #include "count.h"
 #include "dict-common.h"
-#include "disjuncts.h"
 #include "disjunct-utils.h"
 #include "error.h"
 #include "externs.h"
@@ -637,15 +637,15 @@ static void free_post_processing(Sentence sent)
 static void post_process_linkages(Sentence sent, Parse_Options opts)
 {
 	int *indices;
-	int in, block_bottom, block_top;
-	int N_linkages_found, N_linkages_alloced;
-	int N_linkages_post_processed, N_valid_linkages;
-	int overflowed;
+	size_t in, block_bottom, block_top;
+	size_t N_linkages_found, N_linkages_alloced;
+	size_t N_linkages_post_processed, N_valid_linkages;
+	bool overflowed;
 	Linkage_info *link_info;
 #ifdef USE_FAT_LINKAGES
-	int N_thin_linkages;
-	int only_canonical_allowed;
-	int canonical;
+	size_t N_thin_linkages;
+	bool only_canonical_allowed;
+	bool canonical;
 #endif /* USE_FAT_LINKAGES */
 
 	free_post_processing(sent);
@@ -658,7 +658,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		err_ctxt ec;
 		ec.sent = sent;
 		err_msg(&ec, Warn, "Warning: Count overflow.\n"
-		  "Considering a random subset of %d of an unknown and large number of linkages\n",
+		  "Considering a random subset of %zu of an unknown and large number of linkages\n",
 			opts->linkage_limit);
 	}
 	N_linkages_found = sent->num_linkages_found;
@@ -682,8 +682,9 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 		{
 			err_ctxt ec;
 			ec.sent = sent;
-			err_msg(&ec, Warn, "Warning: Considering a random subset of %d of %d linkages\n",
-			        N_linkages_alloced, N_linkages_found);
+			err_msg(&ec, Warn,
+			    "Warning: Considering a random subset of %zu of %zu linkages\n",
+			    N_linkages_alloced, N_linkages_found);
 		}
 	}
 	else
@@ -816,8 +817,8 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 			err_ctxt ec;
 			ec.sent = sent;
 			err_msg(&ec, Error, "Error: None of the linkages is canonical\n"
-			          "\tN_linkages_post_processed=%d "
-			          "N_linkages_found=%d\n",
+			          "\tN_linkages_post_processed=%zu "
+			          "N_linkages_found=%zu\n",
 			          N_linkages_post_processed,
 			          N_linkages_found);
 		}
@@ -827,7 +828,7 @@ static void post_process_linkages(Sentence sent, Parse_Options opts)
 	{
 		err_ctxt ec;
 		ec.sent = sent;
-		err_msg(&ec, Info, "Info: %d of %d linkages with no P.P. violations\n",
+		err_msg(&ec, Info, "Info: %zu of %zu linkages with no P.P. violations\n",
 				N_valid_linkages, N_linkages_post_processed);
 	}
 
@@ -968,7 +969,7 @@ int sentence_split(Sentence sent, Parse_Options opts)
 
 static void free_sentence_words(Sentence sent)
 {
-	int i;
+	size_t i;
 	for (i = 0; i < sent->length; i++)
 	{
 		free_X_nodes(sent->word[i].x);
@@ -1125,7 +1126,7 @@ static inline Boolean is_AFFIXTYPE_EMPTY(const char *a, size_t len)
 
 static void sane_morphism(Sentence sent, Parse_Options opts)
 {
-	int lk, i;
+	size_t lk, i;
 	Parse_info pi = sent->parse_info;
 	Dictionary afdict = sent->dict->affix_table;
 
@@ -1149,7 +1150,7 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 		char affix_types[MAX_SENTENCE+1];/* affix types sequence */
 		char * affix_types_p = affix_types;
 		Boolean match_found = false;     /* djw matched a morpheme */
-		int matched_alts[MAX_SENTENCE];	/* number of morphemes that have matched
+		int matched_alts[MAX_SENTENCE];  /* number of morphemes that have matched
 		                                  * the chosen disjuncts for the
 		                                  * unsplit_word, so far (index: ai) */
 
@@ -1166,12 +1167,12 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 			Boolean empty_word = false;   /* is this an empty word? */
 			Disjunct * cdj = pi->chosen_disjuncts[i];
 
-			lgdebug(+4, ">%d Checking word %d/%d\n", lk, i, sent->length);
+			lgdebug(+4, ">%zu Checking word %zu/%zu\n", lk, i, sent->length);
 
 			/* Ignore island words */
 			if (NULL == cdj)
 			{
-				lgdebug(4, "%d ignored island word\n", lk);
+				lgdebug(4, "%zu ignored island word\n", lk);
 				unsplit = NULL; /* mark it as island */
 				continue;
 			}
@@ -1184,12 +1185,12 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 				numalt = altlen(sent->word[i].alternatives);
 
 				for (ai = 0; ai < numalt; ai++) matched_alts[ai] = 0;
-				lgdebug(4, "%d unsplit word %s, numalt %d\n",
+				lgdebug(4, "%zu unsplit word %s, numalt %d\n",
 						lk, unsplit, (int)numalt);
 			}
 			if (NULL == unsplit)
 			{
-				lgdebug(4, "%d ignore morphemes of an island word\n", lk);
+				lgdebug(4, "%zu ignore morphemes of an island word\n", lk);
 				continue;
 			}
 
@@ -1224,12 +1225,12 @@ static void sane_morphism(Sentence sent, Parse_Options opts)
 				*affix_types_p = AFFIXTYPE_WORD;
 			}
 
-			lgdebug(4, "%d djw=%s affixtype=%c wordlen=%zu\n",
+			lgdebug(4, "%zu djw=%s affixtype=%c wordlen=%zu\n",
 			        lk, djw, empty_word ? 'E' : *affix_types_p, len);
 
 			if (! empty_word) affix_types_p++;
 
-			lgdebug(4, "%d djw %s matched alternatives no.:", lk, djw);
+			lgdebug(4, "%zu djw %s matched alternatives no.:", lk, djw);
 			match_found = false;
 
 			/* Compare the chosen word djw to the alternatives */
@@ -1283,7 +1284,7 @@ try_again:
 					{
 						downcase_utf8_str(downcased, a, MAX_WORD);
 						lgdebug(4, "\n");
-						lgdebug(4, "%d downcasing %s>%s\n", lk, a, downcased);
+						lgdebug(4, "%zu downcasing %s>%s\n", lk, a, downcased);
 						s = downcased;
 						goto try_again;
 					}
@@ -1304,7 +1305,7 @@ try_again:
 			{
 				int num_morphemes = i - unsplit_i + 1;
 
-				lgdebug(4, "%d end of input word, num_morphemes %d\n",
+				lgdebug(4, "%zu end of input word, num_morphemes %d\n",
 						lk, num_morphemes);
 				*affix_types_p++ = AFFIXTYPE_END;
 
@@ -1324,10 +1325,10 @@ try_again:
 
 				if (! match_found)
 				{
-					lgdebug(4, "%d morphemes are missing in this linkage\n", lk);
+					lgdebug(4, "%zu morphemes are missing in this linkage\n", lk);
 					break;
 				}
-				lgdebug(4, "%d Perfect match\n", lk);
+				lgdebug(4, "%zu Perfect match\n", lk);
 			}
 		}
 		*affix_types_p = '\0';
@@ -1347,11 +1348,11 @@ try_again:
 				       "run with verbosity=4 to debug\n", affix_types);
 		}
 		else
-			lgdebug(4, "%d morpheme type combination %s\n", lk, affix_types);
+			lgdebug(4, "%zu morpheme type combination %s\n", lk, affix_types);
 
 		if (match_found)
 		{
-			lgdebug(4, "%d SUCCEEDED\n", lk);
+			lgdebug(4, "%zu SUCCEEDED\n", lk);
 		}
 		else
 		{
@@ -1359,7 +1360,7 @@ try_again:
 			sent->num_valid_linkages --;
 			lifo->N_violations ++;
 			lifo->pp_violation_msg = "Invalid morphism construction.";
-			lgdebug(4, "%d FAILED, remaining %d\n", lk, sent->num_valid_linkages);
+			lgdebug(4, "%zu FAILED, remaining %zu\n", lk, sent->num_valid_linkages);
 		}
 	}
 }
@@ -1389,7 +1390,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 
 		if (verbosity > 1)
 		{
-			prt_error("Info: Total count with %d null links:   %lld\n",
+			prt_error("Info: Total count with %zu null links:   %lld\n",
 				sent->null_count, total);
 		}
 
@@ -1415,7 +1416,7 @@ static void chart_parse(Sentence sent, Parse_Options opts)
 
 static void free_sentence_disjuncts(Sentence sent)
 {
-	int i;
+	size_t i;
 
 	for (i = 0; i < sent->length; ++i)
 	{
@@ -1497,674 +1498,11 @@ int sentence_parse(Sentence sent, Parse_Options opts)
 	if ((verbosity > 0) &&
 	   (PARSE_NUM_OVERFLOW < sent->num_linkages_found))
 	{
-		prt_error("WARNING: Combinatorial explosion! nulls=%d cnt=%d\n"
+		prt_error("WARNING: Combinatorial explosion! nulls=%zu cnt=%d\n"
 			"Consider retrying the parse with the max allowed disjunct cost set lower.\n"
 			"At the command line, use !cost-max\n",
 			sent->null_count, sent->num_linkages_found);
 	}
-	assert(0 <= sent->num_valid_linkages, "Miscount of valid linkages!")
 	return sent->num_valid_linkages;
 }
 
-/***************************************************************
-*
-* Routines which allow user access to Linkages.
-*
-****************************************************************/
-
-Linkage linkage_create(int k, Sentence sent, Parse_Options opts)
-{
-	Linkage linkage;
-
-	if (opts->use_sat_solver)
-	{
-		return sat_create_linkage(k, sent, opts);
-	}
-
-	if ((k >= sent->num_linkages_post_processed) || (k < 0)) return NULL;
-
-	/* Using exalloc since this is external to the parser itself. */
-	linkage = (Linkage) exalloc(sizeof(struct Linkage_s));
-
-	linkage->num_words = sent->length;
-	linkage->word = (const char **) exalloc(linkage->num_words*sizeof(char *));
-	linkage->sent = sent;
-	linkage->opts = opts;
-	linkage->info = &sent->link_info[k];
-#ifdef USE_FAT_LINKAGES
-	linkage->sublinkage = NULL;
-	linkage->unionized = false;
-	linkage->current = 0;
-	linkage->num_sublinkages=0;
-	linkage->dis_con_tree = NULL;
-#endif /* USE_FAT_LINKAGES */
-
-	extract_links(sent->link_info[k].index, sent->parse_info);
-	compute_chosen_words(sent, linkage);
-
-#ifdef USE_FAT_LINKAGES
-	if (set_has_fat_down(sent))
-	{
-		extract_fat_linkage(sent, opts, linkage);
-	}
-	else
-#endif /* USE_FAT_LINKAGES */
-	{
-		extract_thin_linkage(sent, opts, linkage);
-	}
-
-	if (sent->dict->postprocessor != NULL)
-	{
-		linkage_post_process(linkage, sent->dict->postprocessor);
-	}
-
-	return linkage;
-}
-
-int linkage_get_current_sublinkage(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	return linkage->current;
-#else
-	return 0;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_set_current_sublinkage(Linkage linkage, int index)
-{
-#ifdef USE_FAT_LINKAGES
-	if ((index < 0) ||
-		(index >= linkage->num_sublinkages))
-	{
-		return 0;
-	}
-	linkage->current = index;
-#endif /* USE_FAT_LINKAGES */
-	return 1;
-}
-
-static void exfree_pp_info(PP_info *ppi)
-{
-	if (ppi->num_domains > 0)
-		exfree((void *) ppi->domain_name, sizeof(const char *) * ppi->num_domains);
-	ppi->domain_name = NULL;
-	ppi->num_domains = 0;
-}
-
-void linkage_delete(Linkage linkage)
-{
-	int i, j;
-	Sublinkage *s;
-
-	/* Can happen on panic timeout or user error */
-	if (NULL == linkage) return;
-
-	for (i = 0; i < linkage->num_words; ++i)
-	{
-		exfree((void *) linkage->word[i], strlen(linkage->word[i])+1);
-	}
-	exfree((void *) linkage->word, sizeof(const char *) * linkage->num_words);
-
-#ifdef USE_FAT_LINKAGES
-	for (i = 0; i < linkage->num_sublinkages; ++i)
-#endif /* USE_FAT_LINKAGES */
-	{
-#ifdef USE_FAT_LINKAGES
-		s = &(linkage->sublinkage[i]);
-#else
-		s = &(linkage->sublinkage);
-#endif /* USE_FAT_LINKAGES */
-		for (j = 0; j < s->num_links; ++j) {
-			exfree_link(s->link[j]);
-		}
-		exfree(s->link, sizeof(Link*) * s->num_links);
-		if (s->pp_info != NULL) {
-			for (j = 0; j < s->num_links; ++j) {
-				exfree_pp_info(&s->pp_info[j]);
-			}
-			exfree(s->pp_info, sizeof(PP_info) * s->num_links);
-			s->pp_info = NULL;
-			post_process_free_data(&s->pp_data);
-		}
-		if (s->violation != NULL) {
-			exfree((void *) s->violation, sizeof(char) * (strlen(s->violation)+1));
-		}
-	}
-#ifdef USE_FAT_LINKAGES
-	exfree(linkage->sublinkage, sizeof(Sublinkage)*linkage->num_sublinkages);
-#endif /* USE_FAT_LINKAGES */
-
-#ifdef USE_FAT_LINKAGES
-	if (linkage->dis_con_tree)
-		free_DIS_tree(linkage->dis_con_tree);
-#endif /* USE_FAT_LINKAGES */
-	exfree(linkage, sizeof(struct Linkage_s));
-}
-
-#ifdef USE_FAT_LINKAGES
-static int links_are_equal(Link *l, Link *m)
-{
-	return ((l->l == m->l) && (l->r == m->r) && (strcmp(l->name, m->name)==0));
-}
-
-static int link_already_appears(Linkage linkage, Link *link, int a)
-{
-	int i, j;
-
-	for (i=0; i<a; ++i) {
-		for (j=0; j<linkage->sublinkage[i].num_links; ++j) {
-			if (links_are_equal(linkage->sublinkage[i].link[j], link)) return true;
-		}
-	}
-	return false;
-}
-
-static PP_info excopy_pp_info(PP_info ppi)
-{
-	PP_info newppi;
-	int i;
-
-	newppi.num_domains = ppi.num_domains;
-	newppi.domain_name = (const char **) exalloc(sizeof(const char *)*ppi.num_domains);
-	for (i=0; i<newppi.num_domains; ++i)
-	{
-		newppi.domain_name[i] = ppi.domain_name[i];
-	}
-	return newppi;
-}
-
-static Sublinkage unionize_linkage(Linkage linkage)
-{
-	int i, j, num_in_union=0;
-	Sublinkage u;
-	Link *link;
-	const char *p;
-
-	for (i=0; i<linkage->num_sublinkages; ++i) {
-		for (j=0; j<linkage->sublinkage[i].num_links; ++j) {
-			link = linkage->sublinkage[i].link[j];
-			if (!link_already_appears(linkage, link, i)) num_in_union++;
-		}
-	}
-
-	u.link = (Link **) exalloc(sizeof(Link *)*num_in_union);
-	u.num_links = num_in_union;
-	zero_sublinkage(&u);
-
-	u.pp_info = (PP_info *) exalloc(sizeof(PP_info)*num_in_union);
-	u.violation = NULL;
-	u.num_links = num_in_union;
-
-	num_in_union = 0;
-
-	for (i=0; i<linkage->num_sublinkages; ++i) {
-		for (j=0; j<linkage->sublinkage[i].num_links; ++j) {
-			link = linkage->sublinkage[i].link[j];
-			if (!link_already_appears(linkage, link, i)) {
-				u.link[num_in_union] = excopy_link(link);
-				u.pp_info[num_in_union] = excopy_pp_info(linkage->sublinkage[i].pp_info[j]);
-				if (((p=linkage->sublinkage[i].violation) != NULL) &&
-					(u.violation == NULL)) {
-					char *s = (char *) exalloc((strlen(p)+1)*sizeof(char));
-					strcpy(s, p);
-					u.violation = s;
-				}
-				num_in_union++;
-			}
-		}
-	}
-
-	return u;
-}
-#endif /* USE_FAT_LINKAGES */
-
-int linkage_compute_union(Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	int i, num_subs=linkage->num_sublinkages;
-	Sublinkage * new_sublinkage, *s;
-
-	if (linkage->unionized) {
-		linkage->current = linkage->num_sublinkages-1;
-		return 0;
-	}
-	if (num_subs == 1) {
-		linkage->unionized = true;
-		return 1;
-	}
-
-	new_sublinkage =
-		(Sublinkage *) exalloc(sizeof(Sublinkage)*(num_subs+1));
-
-	for (i=0; i<num_subs; ++i) {
-		new_sublinkage[i] = linkage->sublinkage[i];
-	}
-	exfree(linkage->sublinkage, sizeof(Sublinkage)*num_subs);
-	linkage->sublinkage = new_sublinkage;
-
-	/* Zero out the new sublinkage, then unionize it. */
-	s = &new_sublinkage[num_subs];
-	s->link = NULL;
-	s->num_links = 0;
-	zero_sublinkage(s);
-	linkage->sublinkage[num_subs] = unionize_linkage(linkage);
-
-	linkage->num_sublinkages++;
-
-	linkage->unionized = true;
-	linkage->current = linkage->num_sublinkages-1;
-	return 1;
-#else
-	return 0;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_get_num_sublinkages(const Linkage linkage)
-{
-	if (!linkage) return 0;
-#ifdef USE_FAT_LINKAGES
-	return linkage->num_sublinkages;
-#else
-	return 1;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_get_num_words(const Linkage linkage)
-{
-	if (!linkage) return 0;
-	return linkage->num_words;
-}
-
-int linkage_get_num_links(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	int current;
-	if (!linkage) return 0;
-	current = linkage->current;
-	return linkage->sublinkage[current].num_links;
-#else
-	if (!linkage) return 0;
-	return linkage->sublinkage.num_links;
-#endif /* USE_FAT_LINKAGES */
-}
-
-static inline Boolean verify_link_index(const Linkage linkage, int index)
-{
-	if (!linkage) return false;
-	if ((index < 0) ||
-#ifdef USE_FAT_LINKAGES
-		(index >= linkage->sublinkage[linkage->current].num_links))
-#else
-		(index >= linkage->sublinkage.num_links))
-#endif /* USE_FAT_LINKAGES */
-	{
-		return false;
-	}
-	return true;
-}
-
-int linkage_get_link_length(const Linkage linkage, int index)
-{
-	Link *link;
-	int word_has_link[MAX_SENTENCE];
-	int i, length;
-#ifdef USE_FAT_LINKAGES
-	int current = linkage->current;
-#endif /* USE_FAT_LINKAGES */
-
-	if (!verify_link_index(linkage, index)) return -1;
-
-	for (i=0; i<linkage->num_words+1; ++i) {
-		word_has_link[i] = false;
-	}
-
-#ifdef USE_FAT_LINKAGES
-	for (i=0; i<linkage->sublinkage[current].num_links; ++i)
-	{
-		link = linkage->sublinkage[current].link[i];
-		word_has_link[link->l] = true;
-		word_has_link[link->r] = true;
-	}
-	link = linkage->sublinkage[current].link[index];
-#else
-	for (i=0; i<linkage->sublinkage.num_links; ++i)
-	{
-		link = linkage->sublinkage.link[i];
-		word_has_link[link->l] = true;
-		word_has_link[link->r] = true;
-	}
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-
-	length = link->r - link->l;
-	for (i= link->l+1; i < link->r; ++i) {
-		if (!word_has_link[i]) length--;
-	}
-	return length;
-}
-
-int linkage_get_link_lword(const Linkage linkage, int index)
-{
-	Link *link;
-	if (!verify_link_index(linkage, index)) return -1;
-#ifdef USE_FAT_LINKAGES
-	link = linkage->sublinkage[linkage->current].link[index];
-#else
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-	return link->l;
-}
-
-int linkage_get_link_rword(const Linkage linkage, int index)
-{
-	Link *link;
-	if (!verify_link_index(linkage, index)) return -1;
-#ifdef USE_FAT_LINKAGES
-	link = linkage->sublinkage[linkage->current].link[index];
-#else
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-	return link->r;
-}
-
-const char * linkage_get_link_label(const Linkage linkage, int index)
-{
-	Link *link;
-	if (!verify_link_index(linkage, index)) return NULL;
-#ifdef USE_FAT_LINKAGES
-	link = linkage->sublinkage[linkage->current].link[index];
-#else
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-	return link->name;
-}
-
-const char * linkage_get_link_llabel(const Linkage linkage, int index)
-{
-	Link *link;
-	if (!verify_link_index(linkage, index)) return NULL;
-#ifdef USE_FAT_LINKAGES
-	link = linkage->sublinkage[linkage->current].link[index];
-#else
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-	return link->lc->string;
-}
-
-const char * linkage_get_link_rlabel(const Linkage linkage, int index)
-{
-	Link *link;
-	if (!verify_link_index(linkage, index)) return NULL;
-#ifdef USE_FAT_LINKAGES
-	link = linkage->sublinkage[linkage->current].link[index];
-#else
-	link = linkage->sublinkage.link[index];
-#endif /* USE_FAT_LINKAGES */
-	return link->rc->string;
-}
-
-const char ** linkage_get_words(const Linkage linkage)
-{
-	return linkage->word;
-}
-
-Sentence linkage_get_sentence(const Linkage linkage)
-{
-	return linkage->sent;
-}
-
-const char * linkage_get_disjunct_str(const Linkage linkage, int w)
-{
-	Disjunct *dj;
-
-	if (NULL == linkage) return "";
-	if (NULL == linkage->info->disjunct_list_str)
-	{
-		lg_compute_disjunct_strings(linkage->sent, linkage->info);
-	}
-
-	/* dj will be null if the word wasn't used in the parse. */
-	dj = linkage->sent->parse_info->chosen_disjuncts[w];
-	if (NULL == dj) return "";
-
-	return linkage->info->disjunct_list_str[w];
-}
-
-double linkage_get_disjunct_cost(const Linkage linkage, int w)
-{
-	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
-
-	/* dj may be null, if the word didn't participate in the parse. */
-	if (dj) return dj->cost;
-	return 0.0;
-}
-
-double linkage_get_disjunct_corpus_score(const Linkage linkage, int w)
-{
-	Disjunct *dj = linkage->sent->parse_info->chosen_disjuncts[w];
-
-	/* dj may be null, if the word didn't participate in the parse. */
-	if (NULL == dj) return 99.999;
-
-	return lg_corpus_disjunct_score(linkage, w);
-}
-
-const char * linkage_get_word(const Linkage linkage, int w)
-{
-	if (!linkage) return NULL;
-	return linkage->word[w];
-}
-
-int linkage_unused_word_cost(const Linkage linkage)
-{
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0;
-	return linkage->info->unused_word_cost;
-}
-
-double linkage_disjunct_cost(const Linkage linkage)
-{
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0.0;
-	return linkage->info->disjunct_cost;
-}
-
-int linkage_is_fat(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0;
-	return linkage->info->fat;
-#else
-	return false;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_and_cost(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0;
-	return linkage->info->and_cost;
-#else
-	return 0;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_link_cost(const Linkage linkage)
-{
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0;
-	return linkage->info->link_cost;
-}
-
-double linkage_corpus_cost(const Linkage linkage)
-{
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return 0.0;
-	return linkage->info->corpus_cost;
-}
-
-int linkage_get_link_num_domains(const Linkage linkage, int index)
-{
-	PP_info *pp_info;
-	if (!verify_link_index(linkage, index)) return -1;
-#ifdef USE_FAT_LINKAGES
-	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
-#else
-	pp_info = &linkage->sublinkage.pp_info[index];
-#endif /* USE_FAT_LINKAGES */
-	return pp_info->num_domains;
-}
-
-const char ** linkage_get_link_domain_names(const Linkage linkage, int index)
-{
-	PP_info *pp_info;
-	if (!verify_link_index(linkage, index)) return NULL;
-#ifdef USE_FAT_LINKAGES
-	pp_info = &linkage->sublinkage[linkage->current].pp_info[index];
-#else
-	pp_info = &linkage->sublinkage.pp_info[index];
-#endif /* USE_FAT_LINKAGES */
-	return pp_info->domain_name;
-}
-
-const char * linkage_get_violation_name(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	return linkage->sublinkage[linkage->current].violation;
-#else
-	return linkage->sublinkage.violation;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_is_canonical(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return true;
-	return linkage->info->canonical;
-#else
-	return true;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_is_improper(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return false;
-	return linkage->info->improper_fat_linkage;
-#else
-	return false;
-#endif /* USE_FAT_LINKAGES */
-}
-
-int linkage_has_inconsistent_domains(const Linkage linkage)
-{
-#ifdef USE_FAT_LINKAGES
-	/* The sat solver (currently) fails to fill in info */
-	if (!linkage->info) return false;
-	return linkage->info->inconsistent_domains;
-#else
-	return false;
-#endif /* USE_FAT_LINKAGES */
-}
-
-void linkage_post_process(Linkage linkage, Postprocessor * postprocessor)
-{
-#ifdef USE_FAT_LINKAGES
-	int N_sublinkages = linkage_get_num_sublinkages(linkage);
-#endif /* USE_FAT_LINKAGES */
-	Parse_Options opts = linkage->opts;
-	Sentence sent = linkage->sent;
-	Sublinkage * subl;
-	PP_node * pp;
-	int j, k;
-	D_type_list * d;
-
-#ifdef USE_FAT_LINKAGES
-	int i;
-	for (i = 0; i < N_sublinkages; ++i)
-#endif /* USE_FAT_LINKAGES */
-	{
-#ifdef USE_FAT_LINKAGES
-		subl = &linkage->sublinkage[i];
-#else
-		subl = &linkage->sublinkage;
-#endif /* USE_FAT_LINKAGES */
-		if (subl->pp_info != NULL)
-		{
-			for (j = 0; j < subl->num_links; ++j)
-			{
-				exfree_pp_info(&subl->pp_info[j]);
-			}
-			post_process_free_data(&subl->pp_data);
-			exfree(subl->pp_info, sizeof(PP_info) * subl->num_links);
-		}
-		subl->pp_info = (PP_info *) exalloc(sizeof(PP_info) * subl->num_links);
-		for (j = 0; j < subl->num_links; ++j)
-		{
-			subl->pp_info[j].num_domains = 0;
-			subl->pp_info[j].domain_name = NULL;
-		}
-		if (subl->violation != NULL)
-		{
-			exfree((void *)subl->violation, sizeof(char) * (strlen(subl->violation)+1));
-			subl->violation = NULL;
-		}
-
-#ifdef USE_FAT_LINKAGES
-		if (linkage->info->improper_fat_linkage)
-		{
-			pp = NULL;
-		}
-		else
-#endif /* USE_FAT_LINKAGES */
-		{
-			pp = post_process(postprocessor, opts, sent, subl, false);
-			/* This can return NULL, for example if there is no
-			   post-processor */
-		}
-
-		if (pp == NULL)
-		{
-			for (j = 0; j < subl->num_links; ++j)
-			{
-				subl->pp_info[j].num_domains = 0;
-				subl->pp_info[j].domain_name = NULL;
-			}
-		}
-		else
-		{
-			for (j = 0; j < subl->num_links; ++j)
-			{
-				k = 0;
-				for (d = pp->d_type_array[j]; d != NULL; d = d->next) k++;
-				subl->pp_info[j].num_domains = k;
-				if (k > 0)
-				{
-					subl->pp_info[j].domain_name = (const char **) exalloc(sizeof(const char *)*k);
-				}
-				k = 0;
-				for (d = pp->d_type_array[j]; d != NULL; d = d->next)
-				{
-					char buff[5];
-					sprintf(buff, "%c", d->type);
-					subl->pp_info[j].domain_name[k] = string_set_add (buff, sent->string_set);
-
-					k++;
-				}
-			}
-			subl->pp_data = postprocessor->pp_data;
-			if (pp->violation != NULL)
-			{
-				char * s = (char *) exalloc(sizeof(char)*(strlen(pp->violation)+1));
-				strcpy(s, pp->violation);
-				subl->violation = s;
-			}
-		}
-	}
-	post_process_close_sentence(postprocessor);
-}
-
