@@ -12,73 +12,33 @@
 /*************************************************************************/
 
 
-#include <stdarg.h>
+#include <stdint.h>
 
 #include "print.h"
 #include "externs.h"
 #include "api-structures.h"
 #include "corpus/corpus.h"
-#include "idiom.h"
 #include "print-util.h"
 #include "string-set.h"
 #include "structures.h" /* needed for EMPTY_WORD */
 #include "utilities.h"
 #include "word-utils.h"
 
-#define LEFT_WALL_DISPLAY  ("LEFT-WALL")   /* the string to use to show the wall */
 #define LEFT_WALL_SUPPRESS ("Wd") /* If this connector is used on the wall, */
                                   /* then suppress the display of the wall. */
-#define RIGHT_WALL_DISPLAY  ("RIGHT-WALL") /* the string to use to show the wall */
-#define RIGHT_WALL_SUPPRESS ("RW") /* If this connector is used on the wall, */
-
-/* The Russian dictionary makes use of the empty word to deal with
- * the splitting of words into variable-length word-counts */
-#define EMPTY_WORD_SUPPRESS ("ZZZ") /* link to pure whitespace */
-
-#define SUFFIX_WORD ("=")      /* suffixes start with this */
-#define SUFFIX_WORD_L 1        /* length of above */
-#define SUFFIX_SUPPRESS ("LL") /* suffix links start with this */
-#define SUFFIX_SUPPRESS_L 2    /* length of above */
-
-#define HIDE_MORPHO   (!display_morphology)
-
-/**
- * Return TRUE if the word is a suffix.
- *
- * Suffixes have the form =asdf.asdf and "null" suffixes have the form =.asdf.
- * Ordinary equals signs appearing in regular text are either = or =[!].
- */
-static Boolean is_suffix(const char* w)
-{
-	if (0 != strncmp(SUFFIX_WORD, w, SUFFIX_WORD_L)) return FALSE;
-	if (1 == strlen(w)) return FALSE;
-#if SUBSCRIPT_MARK == '.'
-	if (0 == strcmp("=[!]", w)) return FALSE;
-	if (0 == strcmp("=.v", w)) return FALSE;
-	if (0 == strcmp("=.eq", w)) return FALSE;
-#endif
-	return TRUE;
-}
+#define RIGHT_WALL_SUPPRESS ("RW")/* If this connector is used on the wall, */
+                                  /* then suppress the display of the wall. */
 
 static void
 set_centers(const Linkage linkage, int center[],
             Boolean print_word_0, int N_words_to_print)
 {
 	int i, len, tot;
-	Boolean display_morphology = linkage->opts->display_morphology;
 
 	tot = 0;
 	if (print_word_0) i=0; else i=1;
 	for (; i < N_words_to_print; i++)
 	{
-		/* Ignore morphological analysis. */
-		if (HIDE_MORPHO && is_suffix(linkage->word[i]))
-		{
-			center[i] = tot;
-			tot++;  // hack alert -- a trailing blank gets printed anyway ...
-			continue;
-		}
-
 		/* Centers obtained by counting the characters,
 		 * not the bytes in the string.
 		 * len = strlen(linkage->word[i]);
@@ -168,12 +128,14 @@ char * linkage_print_links_and_domains(const Linkage linkage)
 
 	longest = 0;
 	for (link=0; link<N_links; link++) {
-		if (linkage_get_link_lword(linkage, link) == -1) continue;
+		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
+		assert (linkage_get_link_lword(linkage, link) != SIZE_MAX);
 		if (linkage_get_link_num_domains(linkage, link) > longest)
 			longest = linkage_get_link_num_domains(linkage, link);
 	}
 	for (link=0; link<N_links; link++) {
-		if (linkage_get_link_lword(linkage, link) == -1) continue;
+		// if (linkage_get_link_lword(linkage, link) == SIZE_MAX) continue;
+		assert (linkage_get_link_lword(linkage, link) != SIZE_MAX);
 		dname = linkage_get_link_domain_names(linkage, link);
 		for (j=0; j<linkage_get_link_num_domains(linkage, link); ++j) {
 			append_string(s, " (%s)", dname[j]);
@@ -277,8 +239,7 @@ char * linkage_print_disjuncts(const Linkage linkage)
 }
 
 /**
- * XXX TODO: port over the suppression of ZZZ links, and also
- * the HIDE_MORPHO stuff, fom the ascii printing code, way below.
+ * postscript printing ...
  */
 static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *pctx)
 {
@@ -305,8 +266,8 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 		suppressor_used = FALSE;
 		if (!opts->display_walls)
 			for (j=0; j<N_links; j++) {
-				if (ppla[j]->l == 0) {
-					if (ppla[j]->r == linkage->num_words-1) continue;
+				if (ppla[j]->lw == 0) {
+					if (ppla[j]->rw == linkage->num_words-1) continue;
 					N_wall_connectors ++;
 					if (strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS)==0) {
 						suppressor_used = TRUE;
@@ -323,7 +284,7 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	if (dict->right_wall_defined) {
 		suppressor_used = FALSE;
 		for (j=0; j<N_links; j++) {
-			if (ppla[j]->r == linkage->num_words-1) {
+			if (ppla[j]->rw == linkage->num_words-1) {
 				N_wall_connectors ++;
 				if (strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS)==0){
 					suppressor_used = TRUE;
@@ -355,22 +316,23 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	append_string(string,"[");
 	j = 0;
 	for (link=0; link<N_links; link++) {
-		if (!print_word_0 && (ppla[link]->l == 0)) continue;
-		if (!print_word_N && (ppla[link]->r == linkage->num_words-1)) continue;
-		if (ppla[link]->l == -1) continue;
+		if (!print_word_0 && (ppla[link]->lw == 0)) continue;
+		if (!print_word_N && (ppla[link]->rw == linkage->num_words-1)) continue;
+		// if (ppla[link]->lw == SIZE_MAX) continue;
+		assert (ppla[link]->lw != SIZE_MAX);
 		if ((j%7 == 0) && (j>0)) append_string(string,"\n");
 		j++;
-		append_string(string,"[%d %d %d",
-				ppla[link]->l-d, ppla[link]->r-d,
+		append_string(string,"[%zu %zu %d",
+				ppla[link]->lw - d, ppla[link]->rw - d,
 				pctx->link_heights[link]);
 #ifdef USE_FAT_LINKAGES
 		if (ppla[link]->lc->label < 0) {
-			append_string(string," (%s)]", ppla[link]->name);
+			append_string(string," (%s)]", ppla[link]->link_name);
 		} else {
 			append_string(string," ()]");
 		}
 #else
-		append_string(string," (%s)]", ppla[link]->name);
+		append_string(string," (%s)]", ppla[link]->link_name);
 #endif /* USE_FAT_LINKAGES */
 
 	}
@@ -389,145 +351,6 @@ static char * build_linkage_postscript_string(const Linkage linkage, ps_ctxt_t *
 	return ps_string;
 }
 
-/**
- * This takes the current chosen_disjuncts array and uses it to
- * compute the chosen_words array.  "I.xx" suffixes are eliminated.
- */
-void compute_chosen_words(Sentence sent, Linkage linkage)
-{
-	int i, l;
-	char * s, *u;
-	Parse_info pi = sent->parse_info;
-	const char * chosen_words[MAX_SENTENCE];
-	Parse_Options opts = linkage->opts;
-	Boolean display_morphology = opts->display_morphology;
-
-	for (i=0; i<sent->length; i++)
-	{
-		const char *t;
-		/* If chosen_disjuncts is NULL, then this is an 'island' word
-		 * that has not been linked to. */
-		if (pi->chosen_disjuncts[i] == NULL)
-		{
-			/* The unsplit_word is the original word; if its been split
-			 * into stem+suffix, and either one hasn't been choosen, then
-			 * neither should be printed.  Do, however, put brackets around
-			 * the original word, and print that.
-			 */
-			t = sent->word[i].unsplit_word;
-			if (t)
-			{
-				l = strlen(t) + 2;
-				s = (char *) xalloc(l+1);
-				sprintf(s, "[%s]", t);
-				t = string_set_add(s, sent->string_set);
-				xfree(s, l+1);
-			}
-			else
-			{
-				/* Altrnative token island.
-				 * Show the internal word number and its list of alternatives. */
-				String * s = string_new();
-				const char ** a;
-				char * a_list;
-
-				append_string(s, "[%d", i);
-				for (a = sent->word[i].alternatives; *a; a++) {
-					append_string(s, " %s", *a);
-				}
-				append_string(s, "]");
-				a_list = string_copy(s);
-				t = string_set_add(a_list, sent->string_set);
-				string_delete(s);
-				exfree(a_list, strlen(a_list)+1);
-			}
-		}
-		else if (opts->display_word_subscripts)
-		{
-			t = pi->chosen_disjuncts[i]->string;
-			/* get rid of those ugly ".Ixx" */
-			if (is_idiom_word(t)) {
-				s = strdup(t);
-				u = strrchr(s, SUBSCRIPT_MARK);
-				*u = '\0';
-				t = string_set_add(s, sent->string_set);
-				free(s);
-			} else {
-				/* Convert the badly-printing ^C into a period */
-				s = strdup(t);
-				u = strrchr(s, SUBSCRIPT_MARK);
-				if (u) *u = SUBSCRIPT_DOT;
-				t = string_set_add(s, sent->string_set);
-				free(s);
-			}
-
-			/* Suppress the empty word. */
-			if (0 == strcmp(t, EMPTY_WORD_DOT))
-			{
-				t = string_set_add("", sent->string_set);
-			}
-
-			/* Concatenate the stem and the suffix together into one word */
-			if (HIDE_MORPHO)
-			{
-				if (is_suffix(t) && pi->chosen_disjuncts[i-1])
-				{
-					const char * stem = pi->chosen_disjuncts[i-1]->string;
-					size_t len = strlen(stem) + strlen (t);
-					char * join = (char *)malloc(len+1);
-					strcpy(join, stem);
-					u = strrchr(join, SUBSCRIPT_MARK);
-
-					/* u can be null, if the the sentence happens to have
-					 * an equals sign in it, for other reasons. */
-					if (u)
-					{
-						*u = '\0';
-						strcat(join, t + SUFFIX_WORD_L);
-						t = string_set_add(join, sent->string_set);
-					}
-					free(join);
-				}
-
-				/* Suppress printing of the stem, if the next word is the suffix */
-				if ((i+1 < sent->length) &&
-				    (pi->chosen_disjuncts[i+1]))
-				{
-					const char * next = pi->chosen_disjuncts[i+1]->string;
-					if (is_suffix(next) && 0 != strcmp(next, EMPTY_WORD_MARK))
-					{
-						t = string_set_add("", sent->string_set);
-					}
-				}
-			}
-		}
-		else
-		{
-			/* XXX This is wrong, since it fails to indicate what
-			 * was actally used for the parse, which might not actually
-			 * be alternative 0.  We should do like the above, and then
-			 * manually strip the subscript.
-			 * Except that this code is never ever reached, because
-			 * opts->display_word_subscripts is always true...
-			 */
-			t = sent->word[i].alternatives[0];
-		}
-		chosen_words[i] = t;
-	}
-
-	if (sent->dict->left_wall_defined) {
-		chosen_words[0] = LEFT_WALL_DISPLAY;
-	}
-	if (sent->dict->right_wall_defined) {
-		chosen_words[sent->length-1] = RIGHT_WALL_DISPLAY;
-	}
-	for (i=0; i<linkage->num_words; ++i) {
-		s = (char *) exalloc(strlen(chosen_words[i])+1);
-		strcpy(s, chosen_words[i]);
-		linkage->word[i] = s;
-	}
-}
-
 
 #define MAX_HEIGHT 30
 
@@ -547,7 +370,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	char *t;
 	Boolean print_word_0 = 0, print_word_N = 0;
 	int N_wall_connectors, suppressor_used;
-	int center[MAX_SENTENCE];
+	int center[linkage->num_words+1];
 	char connector[MAX_TOKEN_LENGTH];
 	unsigned int line_len, link_length;
 #ifdef USE_FAT_LINKAGES
@@ -568,8 +391,6 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	char xpicture[MAX_HEIGHT][MAX_LINE];
 	size_t start[MAX_HEIGHT];
 
-	Boolean display_morphology = opts->display_morphology;
-
 	string = string_new();
 
 	/* Do we want to print the left wall? */
@@ -581,9 +402,9 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		{
 			for (j=0; j<N_links; j++)
 			{
-				if (0 == ppla[j]->l)
+				if (0 == ppla[j]->lw)
 				{
-					if (ppla[j]->r == linkage->num_words-1) continue;
+					if (ppla[j]->rw == linkage->num_words-1) continue;
 					N_wall_connectors ++;
 					if (0 == strcmp(ppla[j]->lc->string, LEFT_WALL_SUPPRESS))
 					{
@@ -607,7 +428,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 		suppressor_used = FALSE;
 		for (j=0; j<N_links; j++)
 		{
-			if (ppla[j]->r == linkage->num_words-1)
+			if (ppla[j]->rw == linkage->num_words-1)
 			{
 				N_wall_connectors ++;
 				if (0 == strcmp(ppla[j]->lc->string, RIGHT_WALL_SUPPRESS))
@@ -636,29 +457,24 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 	}
 	top_row = 0;
 
-	for (link_length = 1; link_length < N_words_to_print; link_length++) {
-		for (j=0; j<N_links; j++) {
-			if (ppla[j]->l == -1) continue;
-			if (((unsigned int) (ppla[j]->r - ppla[j]->l)) != link_length)
+	for (link_length = 1; link_length < N_words_to_print; link_length++)
+	{
+		for (j=0; j<N_links; j++)
+		{
+			assert (ppla[j]->lw != SIZE_MAX);
+			if (((unsigned int) (ppla[j]->rw - ppla[j]->lw)) != link_length)
 			  continue;
-			if (!print_word_0 && (ppla[j]->l == 0)) continue;
+			if (!print_word_0 && (ppla[j]->lw == 0)) continue;
 			/* gets rid of the irrelevant link to the left wall */
-			if (!print_word_N && (ppla[j]->r == linkage->num_words-1)) continue;
-			/* Get rid of links to empty words */
-			if (0 == strcmp(ppla[j]->name, EMPTY_WORD_SUPPRESS)) continue;
-
-			if (HIDE_MORPHO &&
-			    0 == strncmp(ppla[j]->name, SUFFIX_SUPPRESS, SUFFIX_SUPPRESS_L))
-				continue;
-
-			/* gets rid of the irrelevant link to the right wall */
-			/* ??? */
+			if (!print_word_N && (ppla[j]->rw == linkage->num_words-1)) continue;
 
 			/* put it into the lowest position */
-			cl = center[ppla[j]->l];
-			cr = center[ppla[j]->r];
-			for (row=0; row < MAX_HEIGHT; row++) {
-				for (k=cl+1; k<cr; k++) {
+			cl = center[ppla[j]->lw];
+			cr = center[ppla[j]->rw];
+			for (row=0; row < MAX_HEIGHT; row++)
+			{
+				for (k=cl+1; k<cr; k++)
+				{
 					if (picture[row][k] != ' ') break;
 				}
 				if (k == cr) break;
@@ -680,7 +496,7 @@ static char * linkage_print_diagram_ctxt(const Linkage linkage, ps_ctxt_t *pctx)
 			for (k=cl+1; k<cr; k++) {
 				picture[row][k] = '-';
 			}
-			s = ppla[j]->name;
+			s = ppla[j]->link_name;
 
 			/* display_link_subscripts is always true ... hard-coded */
 			if (opts->display_link_subscripts)
@@ -913,7 +729,7 @@ void linkage_free_pp_msgs(char * s)
 
 void print_disjunct_counts(Sentence sent)
 {
-	int i;
+	size_t i;
 	int c;
 	Disjunct *d;
 	for (i=0; i<sent->length; i++) {
@@ -930,14 +746,14 @@ void print_disjunct_counts(Sentence sent)
 void print_expression_sizes(Sentence sent)
 {
 	X_node * x;
-	int w, size;
+	size_t w, size;
 	for (w=0; w<sent->length; w++) {
 		size = 0;
 		for (x=sent->word[w].x; x!=NULL; x = x->next) {
 			size += size_of_expression(x->exp);
 		}
 		/* XXX alternatives[0] is not really correct, here .. */
-		printf("%s[%d] ",sent->word[w].alternatives[0], size);
+		printf("%s[%zu] ",sent->word[w].alternatives[0], size);
 	}
 	printf("\n\n");
 }
