@@ -695,24 +695,10 @@ int main(int argc, char * argv[])
 
 	for (; i<argc; i++)
 	{
-		if (argv[i][0] == '-')
+		if (argv[i][0] == '-' && strcmp("--version", argv[i]) == 0)
 		{
-			if (strcmp("--version", argv[i])==0)
-			{
-				printf("Version: %s\n", linkgrammar_get_version());
-				exit(0);
-			}
-			/* TBD remove these in version 5.0 */
-			else if ((strcmp("-ppoff", argv[i])==0) ||
-			         (strcmp("-coff", argv[i])==0) ||
-			         (strcmp("-aoff", argv[i])==0))
-			{
-				fprintf(stderr, "%s: Warning: %s flag ignored\n", argv[0], argv[i]);
-			}
-		}
-		else
-		{
-			print_usage(argv[0]);
+			printf("Version: %s\n", linkgrammar_get_version());
+			exit(0);
 		}
 	}
 
@@ -750,23 +736,13 @@ int main(int argc, char * argv[])
 		exit(-1);
 	}
 
-	/* Process the command line like commands */
-	for (i=1; i<argc; i++)
+	/* Process the command line commands */
+	for (i = 1; i<argc; i++)
 	{
-		/* TBD remove these in version 5.0 */
-		if ((strcmp("-pp", argv[i]) == 0) ||
-			(strcmp("-c", argv[i]) == 0) ||
-			(strcmp("-a", argv[i]) == 0) ||
-			(strcmp("-ppoff", argv[i]) == 0) ||
-			(strcmp("-coff", argv[i]) == 0) ||
-			(strcmp("-aoff", argv[i]) == 0))
-		{
-			i++;
-		}
-		else if (argv[i][0] == '-')
+		if (argv[i][0] == '-')
 		{
 			int rc;
-			if (argv[i][1] == '!')
+			if (argv[i][1] == '!' || argv[i][1] == '-')
 				rc = issue_special_command(argv[i]+2, opts, dict);
 			else
 				rc = issue_special_command(argv[i]+1, opts, dict);
